﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.BusinessLogicsContracts;
using CarRepairShopContracts.SearchModels;
using CarRepairShopContracts.StoragesContracts;
using CarRepairShopContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CarRepairShopBusinessLogic.BusinessLogics
{
    public class RepairLogic : IRepairLogic
    {
        private readonly ILogger _logger;
        private readonly IRepairStorage _repairStorage;

        public RepairLogic(ILogger<RepairLogic> logger, IRepairStorage repairStorage)
        {
            _logger = logger;
            _repairStorage = repairStorage;
        }

        public List<RepairViewModel>? ReadList(RepairSearchModel? model)
        {
            _logger.LogInformation("ReadList. RepairName:{RepairName}. Id:{Id}", model?.RepairName, model?.Id);
            var list = model == null ? _repairStorage.GetFullList() : _repairStorage.GetFilteredList(model);
            if(list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public RepairViewModel? ReadElement(RepairSearchModel model)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. RepairName:{RepairName}. Id:{Id}", model.RepairName, model.Id);
            var element = _repairStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(RepairBindingModel model)
        {
            CheckModel(model);
            if (_repairStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(RepairBindingModel model)
        {
            CheckModel(model, false);
            if(_repairStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public bool Update(RepairBindingModel model)
        {
            CheckModel(model);
            if(_repairStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(RepairBindingModel model, bool withParams = true)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if(string.IsNullOrEmpty(model.RepairName))
            {
                throw new ArgumentNullException("Нет названия продукта", nameof(model.RepairName));
            }
            if(model.Price <= 0)
            {
                throw new ArgumentNullException("Цена продукта должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Repair. RepairName:{RepairName}. Price:{Price}. Id:{Id}", model.RepairName, model.Price, model.Id);
            var element = _repairStorage.GetElement(new RepairSearchModel
            {
                RepairName = model.RepairName
            });
            if(element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Продукт с таким названием уже есть");
            }
        }
    }
}
